Package: AnnotationDbi
Title: Manipulation of SQLite-based annotations in Bioconductor
Description: Implements a user-friendly interface for querying SQLite-based annotation data packages.
biocViews: Annotation, Microarray, Sequencing, GenomeAnnotation
URL: https://bioconductor.org/packages/AnnotationDbi
Video: https://www.youtube.com/watch?v=8qvGNTVz3Ik
BugReports: https://github.com/Bioconductor/AnnotationDbi/issues
Version: 1.66.0
License: Artistic-2.0
Encoding: UTF-8
Author: Hervé Pagès, Marc Carlson, Seth Falcon, Nianhua Li
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Depends: R (>= 2.7.0), methods, stats4, BiocGenerics (>= 0.29.2),
        Biobase (>= 1.17.0), IRanges
Imports: DBI, RSQLite, S4Vectors (>= 0.9.25), stats, KEGGREST
Suggests: utils, hgu95av2.db, GO.db, org.Sc.sgd.db, org.At.tair.db,
        RUnit, TxDb.Hsapiens.UCSC.hg19.knownGene, org.Hs.eg.db,
        reactome.db, AnnotationForge, graph, EnsDb.Hsapiens.v75,
        BiocStyle, knitr
VignetteBuilder: knitr
Collate: 00RTobjs.R AllGenerics.R AllClasses.R unlist2.R utils.R SQL.R
        FlatBimap.R AnnDbObj-lowAPI.R Bimap.R GOTerms.R
        BimapFormatting.R Bimap-envirAPI.R flatten.R
        methods-AnnotationDb.R methods-SQLiteConnection.R
        methods-geneCentricDbs.R methods-geneCentricDbs-keys.R
        methods-ReactomeDb.R methods-OrthologyDb.R loadDb.R
        createAnnObjs-utils.R createAnnObjs.NCBIORG_DBs.R
        createAnnObjs.NCBICHIP_DBs.R createAnnObjs.ORGANISM_DB.R
        createAnnObjs.YEASTCHIP_DB.R createAnnObjs.COELICOLOR_DB.R
        createAnnObjs.ARABIDOPSISCHIP_DB.R createAnnObjs.MALARIA_DB.R
        createAnnObjs.YEAST_DB.R createAnnObjs.YEASTNCBI_DB.R
        createAnnObjs.ARABIDOPSIS_DB.R createAnnObjs.GO_DB.R
        createAnnObjs.KEGG_DB.R createAnnObjs.PFAM_DB.R
        AnnDbPkg-templates-common.R AnnDbPkg-checker.R
        print.probetable.R makeMap.R inpIDMapper.R
        test_AnnotationDbi_package.R
git_url: https://git.bioconductor.org/packages/AnnotationDbi
git_branch: RELEASE_3_19
git_last_commit: 989c1dc
git_last_commit_date: 2024-04-30
Repository: Bioconductor 3.19
Date/Publication: 2024-05-01
NeedsCompilation: no
Packaged: 2024-05-01 20:00:57 UTC; biocbuild
Built: R 4.4.0; ; 2024-05-07 01:57:59 UTC; unix
