import(methods)
import(AnnotationDbi)

### Only put what is statically exported here. All the AnnObj instances
### created at load time are dynamically exported (refer to R/zzz.R for
### the details).
export(
    org.testing,
    org.testing_dbconn,
    org.testing_dbfile,
    org.testing_dbschema,
    org.testing_dbInfo,

    org.testingORGANISM
)

