\name{org.TguttataTestingSubset.egORGANISM}
\alias{org.TguttataTestingSubset.egORGANISM}
\title{The Organism for org.TguttataTestingSubset.eg}
\description{
  org.TguttataTestingSubset.egORGANISM is an R object that contains a single item: a
  character string that names the organism for which org.TguttataTestingSubset.eg was built.
}
\details{
  Although the package name is suggestive of the organism for which it
  was built, org.TguttataTestingSubset.egORGANISM provides a simple way to programmatically
  extract the organism name.
}
\examples{
  org.TguttataTestingSubset.egORGANISM
}
\keyword{datasets}

